import GObject from "gi://GObject";
import Gvc from "gi://Gvc";
import * as Volume from "resource:///org/gnome/shell/ui/status/volume.js";
const { BindingFlags } = GObject;
class AudioManagerClass extends GObject.Object {
    _control;
    _connections;
    constructor() {
        super();
        this._control = Volume.getMixerControl();
        this._connections = [];
        this._connections.push(this._control.connect("stream-added", (control, id) => {
            const stream = control.lookup_stream_id(id);
            if (stream instanceof Gvc.MixerSinkInput) {
                this.emit("application-output-added", new ApplicationOutput(this, stream));
            }
        }));
        this._connections.push(this._control.connect("stream-removed", (control, id) => {
            const stream = control.lookup_stream_id(id);
            if (stream instanceof Gvc.MixerSinkInput) {
                this.emit("application-output-removed", new ApplicationOutput(this, stream));
            }
        }));
    }
    get application_outputs() {
        return this._control.get_sink_inputs().map(stream => new ApplicationOutput(this, stream));
    }
    vfunc_dispose() {
        super.vfunc_dispose();
        for (const connection of this._connections) {
            this._control.disconnect(connection);
        }
        this._connections.length = 0;
        // @ts-expect-error
        delete this._control;
    }
}
export const AudioManager = GObject.registerClass({
    Signals: {
        "application-output-added": {
            // @ts-expect-error
            param_types: [AudioManagerClass],
        },
        "application-output-removed": {
            // @ts-expect-error
            param_types: [AudioManagerClass],
        },
    },
}, AudioManagerClass);
export const ApplicationOutput = GObject.registerClass({
    Properties: {
        "name": GObject.ParamSpec.string("name", "Name", "The name of the stream, typically the name of the associated application", GObject.ParamFlags.READWRITE, null),
        "description": GObject.ParamSpec.string("description", "Description", "The description of the stream, typically the name of the playing file or video", GObject.ParamFlags.READWRITE, null),
        "icon_name": GObject.ParamSpec.string("icon_name", "Icon name", "", GObject.ParamFlags.READWRITE, null),
    },
}, class ApplicationOutput extends GObject.Object {
    name;
    description;
    icon_name;
    _manager;
    _stream;
    constructor(manager, stream) {
        super();
        this._manager = manager;
        this._stream = stream;
        this._stream.bind_property("name", this, "name", BindingFlags.SYNC_CREATE | BindingFlags.BIDIRECTIONAL);
        this._stream.bind_property("description", this, "description", BindingFlags.SYNC_CREATE | BindingFlags.BIDIRECTIONAL);
        this._stream.bind_property("icon_name", this, "icon_name", BindingFlags.SYNC_CREATE | BindingFlags.BIDIRECTIONAL);
    }
    get id() {
        return this._stream.id;
    }
});
